<?php

require_once(dirname(__FILE__).'/config.php');
require_once(dirname(__FILE__).'/migrator.php');
require_once(dirname(__FILE__).'/postmeta.php');
require_once(dirname(__FILE__).'/php.php');

// ----------------------------------------------------------------------------
// the DiscloseSecret_Option class handles the loading and storing of the
// plugin option including all needed conversion routines
// ----------------------------------------------------------------------------

class DiscloseSecret_Option
{
	var $m_version = DiscloseSecret_VERSION;
	var $m_caps = array();
	var $m_roles = array();
	var $m_users = array();
	var $m_feed_options = array();
	var $m_feed_default_option;

	// default option values will be set during initialization
	var $m_feed_privacy;
	var $m_feed_teaser;
	var $m_rewrite_feed_link;

	// ----------------------------------------------------------------------------
	// init
	// ----------------------------------------------------------------------------

	function DiscloseSecret_Option()
	{
		$this->init_constants();
		$this->init();
	}

	// initializes plugin data
	function init_constants()
	{
		global $wpdb, $wp_roles;

		// init user list first, to init role system
		$query = "SELECT `ID` AS user_id, `display_name` FROM `{$wpdb->users}` ORDER BY `display_name`";
		$wpdb->query($query);
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();
		foreach ($s as $i)
		{
			$this->m_users[$i->user_id] = array(
				DiscloseSecret_INDEX_ID                => $i->user_id,
				DiscloseSecret_INDEX_NAME              => $i->display_name,
				DiscloseSecret_INDEX_CAN_SEE_PROTECTED => $this->user_can_see_protected($i->user_id),
				DiscloseSecret_INDEX_CAN_SEE_IMPLICIT  => false
			);
		}

		// paranoidly force init of role system again by starting a dummy request
		get_role('Administrator');
		foreach ($wp_roles->role_objects as $role)
		{
			$this->m_roles[$role->name] = array(
				DiscloseSecret_INDEX_ID                => $role->name,
				DiscloseSecret_INDEX_NAME              => ucwords(str_replace('-', ' ', str_replace('_', ' ', $role->name))),
				DiscloseSecret_INDEX_CAN_SEE_PROTECTED => false,
				DiscloseSecret_INDEX_CAN_SEE_IMPLICIT  => false
			);
			foreach ($role->capabilities as $cap => $grant)
			{
				$this->m_caps[$cap] = array(
					DiscloseSecret_INDEX_ID                => $cap,
					DiscloseSecret_INDEX_NAME              => ucwords(str_replace('-', ' ',str_replace('_', ' ', $cap))),
					DiscloseSecret_INDEX_CAN_SEE_PROTECTED => ($cap == 'edit_others_posts'),
					DiscloseSecret_INDEX_CAN_SEE_IMPLICIT  => false
				);
				if ($this->m_caps[$cap][DiscloseSecret_INDEX_CAN_SEE_PROTECTED])
					$this->m_roles[$role->name][DiscloseSecret_INDEX_CAN_SEE_PROTECTED] = true;
			}
		}
		ksort($this->m_caps);
		ksort($this->m_roles);

		// init constants
		$this->m_feed_options = array(
			DiscloseSecret_OPTION_FULL_CONTENT => __('Full Content', DiscloseSecret_PLUGIN_ID),
			DiscloseSecret_OPTION_EXCERPT_ONLY => __('Excerpt Only', DiscloseSecret_PLUGIN_ID),
			DiscloseSecret_OPTION_TITLE_ONLY   => __('Title Only', DiscloseSecret_PLUGIN_ID)
		);
		$this->m_feed_default_option = __('Default', DiscloseSecret_PLUGIN_ID);
	}

	// Loads plugin option and probably upgrade older versions of the plugin
	function init()
	{
		$option = get_option(DiscloseSecret_PLUGIN_ID);
		if ($option === false)
		{
			$option = get_option(DiscloseSecret_POSTRESTRICTIONS_PLUGIN_ID);
			if ($option === false)
				$version = '0';
			else
				$version = '1.2';
		}
		else
			$version = $option[DiscloseSecret_OPTION_VERSION];
		$this->upgrade($version);
		$this->load();
	}

	// ----------------------------------------------------------------------------
	// option handling
	// ----------------------------------------------------------------------------

	// Upgrades plugin from previous versions or even installs it
	function upgrade($old_version)
	{
		while ($old_version != DiscloseSecret_VERSION)
		{
			$this->load();
			if ($old_version == '0')
			{
				// this is first installation of the plugin
				$old_version = '1.2';
			}
			else if ($old_version == '1.2')
			{
				// migrate predecessor restriction settings
				$old_version = '2.0';
				$migrator = new DiscloseSecret_Migrator();
				$migrator->migrate_post_restrictions(true);
				// just reload options after plugin name change
				$this->load();
			}
			else if ($old_version == '2.0')
			{
				// migrate predecessor restriction settings
				$old_version = '2.0.1';
			}
			else
			{
				die(sprintf(__('No upgrade path. This is version %s of the plugin, previously version %s was.', DiscloseSecret_PLUGIN_ID),
					DiscloseSecret_VERSION, $old_version));
			}
			$this->m_version = $old_version;
			$this->save();
			$this->m_version = DiscloseSecret_VERSION;
		}
	}

	function save()
	{
		// introduced in 2.0 but we save it anyway during migration
		$option[DiscloseSecret_OPTION_VERSION] = $this->m_version;

		// always in
		$option[DiscloseSecret_OPTION_FEED_PRIVACY] = $this->m_feed_privacy;

		// introduced in 2.0
		if (substr($this->m_version, 0, 2) == '2.')
			$option[DiscloseSecret_OPTION_FEED_TEASER] = $this->m_feed_teaser;

		// always in
		$option[DiscloseSecret_OPTION_REWRITE_FEED_LINK] = $this->m_rewrite_feed_link;

		update_option(DiscloseSecret_PLUGIN_ID, $option);
	}

	function load()
	{
		$option = get_option(DiscloseSecret_PLUGIN_ID);

		// always in
		if (isset($option[DiscloseSecret_OPTION_FEED_PRIVACY]))
			$this->m_feed_privacy = $option[DiscloseSecret_OPTION_FEED_PRIVACY];
		else
			$this->m_feed_privacy = DiscloseSecret_OPTION_TITLE_ONLY;

		// always in
		if (isset($option[DiscloseSecret_OPTION_REWRITE_FEED_LINK]))
			$this->m_rewrite_feed_link = $option[DiscloseSecret_OPTION_REWRITE_FEED_LINK];
		else
			$this->m_rewrite_feed_link = true;

		// introduced in 2.0
		if (isset($option[DiscloseSecret_OPTION_FEED_TEASER]))
			$this->m_feed_teaser = $option[DiscloseSecret_OPTION_FEED_TEASER];
		else
			$this->m_feed_teaser = __("View the content of this <a href='%permalink%'>protected post</a>", DiscloseSecret_PLUGIN_ID);
	}

	// ----------------------------------------------------------------------------
	// tools
	// ----------------------------------------------------------------------------

	function user_can_see_protected($user_id)
	{
		$user = new WP_User($user_id);
		if ($user->has_cap('edit_others_posts'))
			return true;
		return false;
	}

	// ----------------------------------------------------------------------------
	// access
	// ----------------------------------------------------------------------------

	function set_from_POST()
	{
		$this->m_feed_privacy = isset($this->m_feed_options[$_POST[DiscloseSecret_POST_FEED_PRIVACY]])
			? $_POST[DiscloseSecret_POST_FEED_PRIVACY] : DiscloseSecret_OPTION_TITLE_ONLY;
		$this->m_feed_teaser
			= stripslashes(htmlspecialchars_decode($_POST[DiscloseSecret_POST_FEED_TEASER], ENT_QUOTES));
		$this->m_rewrite_feed_link
			= isset($_POST[DiscloseSecret_POST_REWRITE_FEED_LINK]);
	}

	function set_can_see_implicit($postmeta)
	{
		global $wp_roles;

		foreach (array_keys($this->m_caps) as $key)
		{
			$this->m_caps[$key][DiscloseSecret_INDEX_CAN_SEE_IMPLICIT] = false;
		}

		foreach (array_keys($this->m_roles) as $key)
		{
			$implicit = false;
			$r = $wp_roles->get_role($this->m_roles[$key][DiscloseSecret_INDEX_ID]);
			foreach ($postmeta->get_caps() as $c)
			{
				if ($r->has_cap($c))
				{
					$implicit = true;
					break;
				}
			}
			$this->m_roles[$key][DiscloseSecret_INDEX_CAN_SEE_IMPLICIT] = $implicit;
		}

		foreach (array_keys($this->m_users) as $key)
		{
			$implicit = false;
			$u = new WP_User($this->m_users[$key][DiscloseSecret_INDEX_ID]);
			foreach ($postmeta->get_caps() as $c)
			{
				if ($u->has_cap($c))
				{
					$implicit = true;
					break;
				}
			}
			if (!$implicit)
			{
				foreach ($postmeta->get_roles() as $r)
				{
					if ($u->has_cap($r))
					{
						$implicit = true;
						break;
					}
				}
			}
			$this->m_users[$key][DiscloseSecret_INDEX_CAN_SEE_IMPLICIT] = $implicit;
		}
	}

	function &get_caps()
	{
		return $this->m_caps;
	}

	function &get_roles()
	{
		return $this->m_roles;
	}

	function &get_users()
	{
		return $this->m_users;
	}

	function get_feed_options()
	{
		return $this->m_feed_options;
	}

	function get_feed_default_option()
	{
		return $this->m_feed_default_option;
	}

	function get_feed_privacy()
	{
		return $this->m_feed_privacy;
	}

	function get_feed_teaser()
	{
		return $this->m_feed_teaser;
	}

	function get_rewrite_feed_link()
	{
		return $this->m_rewrite_feed_link;
	}
}

?>